// Updates: 2004.09.30


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


/**
 * An on/off button represented by a traffic light.
 * @author Michel Deriaz
 */
public class Light extends JPanel {
  private final int MIN = 100; // intensity of the light, when off
  private Vector listeners;
  private int red, green;
  
  
  /**
   * Creates a traffic light that will change state each time a user clicks on
   * it.
   */  
  public Light() {
    this(true);
  }
  
  
  /**
   * Creates a traffic light that works according to the parameter: if 
   * <code>automatic == true</code> the light will change its state each time a
   * users clicks on it; if <code>automatic == false</code> the light will not 
   * change it's state.
   * @param automatic tells whether the light must automatically change its 
   * state each time a user clicks on it
   */
  public Light(final boolean automatic) {
    listeners = new Vector();
    addMouseListener(new MouseAdapter() {
      public void mouseReleased(MouseEvent e) {
        if (automatic) change();
        fireLightListener();
      }      
    });    
    setPreferredSize(new Dimension(14, 26));
    red = 255;
    green = MIN;
  }
  
  
  /**
   * Adds a <code>LightListener</code> to the traffic light that will be
   * notified each time a user clicks on the light.
   * @param listener the LightListener to be added
   */
  public void addLightListener(LightListener listener) {
    listeners.add(listener);
  }
  

  /**
   * Removes a <code>LightListener</code> from the traffic light.
   * @param listener the LightListener to be removed
   */
  public void removeLightListener(LightListener listener) {
    listeners.remove(listener);
  }  
  

  /**
   * Sets the light to red.
   */  
  public void setRed() {
    red = 255;
    green = MIN;
    repaint();
  }
  
  
  /**
   * Sets the light to green.
   */    
  public void setGreen() {
    red = MIN;
    green = 255;
    repaint();
  }
  
  
  /**
   * Changes the state of the light.
   */     
  public void change() {
    red = MIN + 255 - red;
    green = MIN + 255 - green;
    repaint();
  }
  
  
  /**
   * Returns whether the light is red.
   * @return whether the light is red
   */   
  public boolean isRed() {
    return (red == 255);
  }
  
  
  /**
   * Returns whether the light is green.
   * @return whether the light is green
   */   
  public boolean isGreen() {
    return (green == 255);
  }

  
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    ((Graphics2D)g).setRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
    g.setColor(Color.BLACK);
    g.fillRoundRect(0, 0, 14, 26, 10, 10);
    g.setColor(new Color(red, 0, 0));
    g.fillOval(2, 2, 10, 10);
    g.setColor(new Color(0, green, 0));
    g.fillOval(2, 14, 10, 10);    
  }


  private void fireLightListener() {
    for (int i = 0; i < listeners.size(); i++) {
      ((LightListener)listeners.elementAt(i)).lightClicked();
    }
  }  
}